#ifndef _DATABASE_H
#define _DATABASE_H

#include "db_access.h"

#define MAX_LIST_ITEMS		9
#define MAX_EMAIL_LEN		80
#define MAX_EMAILSTR_LEN	(MAX_LIST_ITEMS * (MAX_EMAIL_LEN + 1) + 1)
#define MAX_FIELD_LEN		81


typedef struct {
	char *key;
	char *name;
	int type;
} abook_field;

typedef struct {
	int count;
	abook_field **fields;
} abook_field_list;

enum {
	FIELD_STRING = 1,
	FIELD_EMAILS,
	FIELD_LIST,
	FIELD_DAY,
};


/*
 * Field operations
 */
abook_field *find_standard_field(char *key, int do_declare);
abook_field *real_find_field(char *key, abook_field_list *list, int *nb);
#define find_field(key, list)		real_find_field(key, list, NULL)
#define find_field_number(key, pt_nb)	real_find_field(key, NULL, pt_nb)
#define find_declared_field(key)	find_field(key,NULL)
void add_field(abook_field_list *list, abook_field *f);
char *declare_new_field(char *key, char *name, char *type, int accept_standard);
void init_standard_fields();

/*
 * Various database operations
 */
void prepare_database_internals();
int load_database(char *filename);
int save_database();
void adjust_list_capacity();
void remove_selected_items();
void sort_surname();
void sort_by_field(char *field);
void close_database();
int find_item(char *str, int start, int search_fields[]);
int is_selected(int item);
int is_valid_item(int item);
int last_item();


/*
 * database item read
 */
list_item db_item_get(int i);



#endif /* _DATABASE_H */

