#ifndef _FILTER_FILTER_INTERFACE_H
#define _FILTER_FILTER_INTERFACE_H

#include "../db_access.h"
#include "../options.h"

/*
 * Interface available for filter modules.
 *
 * Any access to the database, items, fields and options _must_ go through
 * this interface.
 */
struct abook_ops {
	/*
	 * Database read operations.
	 *
	 *   within filters, use:
	 *       db_fget(),
	 *       db_fgetbyid(),
	 *       db_name_get(),
	 *       db_email_get(),
	 *       get_first_email(),
	 */
	char *	(*real_db_field_get)	(int, int, int);
	char *	(*real_get_first_email)	(int);
	/*
	 * Database write operations.
	 *
	 *   within filters, use:
	 *       db_fput(),
	 *       db_fputbyid(),
	 *       add_item2database()
	 */
	int 	(*real_db_field_put)	(int, int, int, char *);
	int	(*real_add_item2database)	(list_item);
	/*
	 * Item operations.
	 *
	 *   within filters, use:
	 *       item_create(),
	 *       item_resize(),
	 *       item_free(),
	 *       item_empty(),
	 *       item_blank(),
	 *       item_copy(),
	 *       item_duplicate(),
	 *       item_fput(),
	 *       item_fget()
	 */
	list_item	(*item_do)	(enum item_ops,
			list_item, list_item, list_item *);
	int 	(*real_item_fput)	(list_item, int, char *);
	char *	(*real_item_fget)	(list_item, int);
	/*
	 * Traversal operations.
	 *
	 *   within filters, use:
	 *       e = init_db_enumerator(),
	 *       e = init_field_enumerator(),
	 *     and then
	 *       db_enumerate_items(e) { ... },
	 *       enumerate_fields(e) { ... }
	 */
	int	(*real_db_enumerate_items)	(struct db_enumerator e);
	int	(*real_enumerate_fields)	(struct field_enumerator *e);
	/*
	 * Fields operations.
	 *
	 *   within filters, use:
	 *       field_id(),
	 *       field_key(),
	 *       field_name(),
	 *       field_type(),
	 *       field_index(),
	 *       declare_unknown_field()
	 */
	int	(*real_field_id)	(int);
	union ret_value	(*field_query)	(enum field_ops, int, char *);
	int	(*real_declare_unknown_field)	(char *);
	/*
	 * Miscellaneous.
	 *
	 *   within filters, use:
	 *       opt_get_str(),
	 *       opt_get_int(),
	 *       opt_get_bool()
	 */
	union opt_value	(*opt_get_value)	(enum opt_type type, int opt);
};

/*
 * Interface provided by filter modules to abook.
 */
struct filter_ops {
	int (*export)(FILE *, struct db_enumerator);
	int (*import)(FILE *);
	char *(*description)();
};

#endif /* _FILTER_FILTER_INTERFACE_H */
