
/*
 * $Id$
 *
 * (C) by abook developers
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#ifdef HAVE_CONFIG_H
#	include "../config.h"
#endif
#include "../abook.h"
#include "../db_access.h"
#include "../misc.h"
#include "../xmalloc.h"

#include "filter_interface.h"
#include "filter.h"


/*
 * mutt alias import
 */

#include "../getname.h"

static int
mutt_read_line(FILE *in, char **alias, char **rest)
{
	char *line, *ptr, *tmp;
	size_t alias_len;

	if( !(line = ptr = getaline(in)) )
		return 1; /* error / EOF */

	SKIPWS(ptr);

	if(strncmp("alias", ptr, 5)) {
		free(line);
		return 1;
	}

	ptr += 5;

	SKIPWS(ptr);

	tmp = ptr;

	while( ! ISSPACE(*ptr) )
		ptr++;

	alias_len = (size_t)(ptr - tmp);

	if(alias)
		*alias = xmalloc_inc(alias_len, 1);

	strncpy(*alias, tmp, alias_len);
	*(*alias + alias_len) = 0;

	SKIPWS(ptr);

	*rest = xstrdup(ptr);

	free(line);
	return 0;
}

static void
mutt_fix_quoting(char *p)
{
	char *escape = 0;

	for(; *p; p++) {
		switch(*p) {
			case '\"':
				if(escape)
					*escape = ' ';
				break;
			case '\\':
				escape = p;
				break;
			default:
				escape = 0;
		}
	}
}

static void
mutt_parse_email(list_item item)
{
	char *line = item_fget(item, NAME);
	char *tmp;
	char *name, *email;
#if 0
	char *start = line;
	int i = 0;
#endif

	mutt_fix_quoting(line);
	tmp = strconcat("From: ", line, NULL);
	getname(tmp, &name, &email);
	free(tmp);

	if(name)
		item_fput(item, NAME, name);
	else
		return;

	if(email)
		item_fput(item, EMAIL, email);
	else
		return;

	/*
	 * this is completely broken
	 */
#if 0
	while( (start = strchr(start, ',')) && i++ < MAX_EMAILS - 1) {
		tmp = strconcat("From: ", ++start, NULL);
		getname(tmp, &name, &email);
		free(tmp);
		free(name);
		if(email) {
			if(*email) {
				tmp = strconcat(item[EMAIL], ",", email, NULL);
				free(item[EMAIL]);
				item[EMAIL] = tmp;
			} else {
				xfree(email);
			}
		}
	}
#endif
}

static int
mutt_parse_file(FILE *in)
{
	list_item item = item_create();

	for(;;) {
		item_blank(item);

		if(!mutt_read_line(in,
					(field_id(NICK) != -1) ?
					&item[field_id(NICK)] :	NULL,
					&item[field_id(NAME)]))
			mutt_parse_email(item);

		if(feof(in)) {
			item_empty(item);
			break;
		}

		add_item2database(item);
	}
	item_free(&item);

	return 0;
}

/*
 * end of mutt alias import
 */

/*
 * mutt alias export filter
 */

static char *
mutt_alias_genalias(int i)
{
	char *tmp, *pos;

	if(db_fget(i, NICK))
		return xstrdup(db_fget(i, NICK));

	tmp = xstrdup(db_name_get(i));

	if( ( pos = strchr(tmp, ' ') ) )
		*pos = 0;

	strlower(tmp);

	return tmp;
}

static int
mutt_alias_export(FILE *out, struct db_enumerator e)
{
	char *email;
	char *alias = NULL;

	db_enumerate_items(e) {
		alias = mutt_alias_genalias(e.item);
		email = get_first_email(e.item);
		fprintf(out, *email ? "alias %s %s <%s>\n": "alias %s %s%s\n",
				alias,
				db_name_get(e.item),
				email);
		xfree(alias);
		xfree(email);
	}

	return 0;
}

/*
 * end of mutt alias export
 */



/*
 * Using filter helper
 */
#define FILTER_NAME mutt
#define FILTER_DESCRIPTION N_("mutt alias")
#define FILTER_IMPORT_FUNCTION mutt_parse_file
#define FILTER_EXPORT_FUNCTION mutt_alias_export
#include "filter_helper.h"
